@@ -50,18 +50,22 @@
 
 #include "qwebdav.h"
 
+#include <QDebug>
+#include <QTextStream>
+
+Q_LOGGING_CATEGORY(KDAV2_LOG, "org.kde.pim.kdav2.webdav")
+
 QWebdav::QWebdav (QObject *parent) : QNetworkAccessManager(parent)
   ,m_rootPath()
   ,m_username()
   ,m_password()
   ,m_baseUrl()
   ,m_currentConnectionType(QWebdav::HTTP)
-  ,m_authenticator_lastReply(0)
+  ,m_authenticator_lastReply(nullptr)
 
 {
     qRegisterMetaType<QNetworkReply*>("QNetworkReply*");
 
-    connect(this, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
     connect(this, SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)), this, SLOT(provideAuthenication(QNetworkReply*,QAuthenticator*)));
     connect(this, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)), this, SLOT(sslErrors(QNetworkReply*,QList<QSslError>)));
 }
@@ -149,71 +153,9 @@ void QWebdav::setConnectionSettings(const QWebdavConnectionType connectionType,
     m_password = password;
 }
 
-void QWebdav::replyReadyRead()
-{
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(QObject::sender());
-    if (reply->bytesAvailable() < 256000)
-        return;
-
-    QIODevice* dataIO = m_inDataDevices.value(reply, 0);
-    if(dataIO == 0)
-        return;
-    dataIO->write(reply->readAll());
-}
-
-void QWebdav::replyFinished(QNetworkReply* reply)
-{
-    qDebug() << "QWebdav::replyFinished() " << reply->rawHeaderList();
-
-    disconnect(reply, SIGNAL(readyRead()), this, SLOT(replyReadyRead()));
-    disconnect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(replyError(QNetworkReply::NetworkError)));
-
-    QIODevice* dataIO = m_inDataDevices.value(reply, 0);
-    if (dataIO != 0) {
-        dataIO->write(reply->readAll());
-        static_cast<QFile*>(dataIO)->flush();
-        dataIO->close();
-        delete dataIO;
-    }
-    m_inDataDevices.remove(reply);
-
-    QMetaObject::invokeMethod(this,"replyDeleteLater", Qt::QueuedConnection, Q_ARG(QNetworkReply*, reply));
-}
-
-void QWebdav::replyDeleteLater(QNetworkReply* reply)
-{
-    qDebug() << "QWebdav::replyDeleteLater()";
-
-    QIODevice *outDataDevice = m_outDataDevices.value(reply, 0);
-    if (outDataDevice!=0)
-        outDataDevice->deleteLater();
-    m_outDataDevices.remove(reply);
-}
-
-void QWebdav::replyError(QNetworkReply::NetworkError)
-{
-    QNetworkReply* reply = qobject_cast<QNetworkReply*>(QObject::sender());
-    if (reply==0)
-        return;
-
-    qWarning() << "QWebdav::replyError()  reply->url() == " << reply->url().toString(QUrl::RemoveUserInfo);
-
-    if ( reply->error() == QNetworkReply::OperationCanceledError) {
-        QIODevice* dataIO = m_inDataDevices.value(reply, 0);
-        if (dataIO!=0) {
-            delete dataIO;
-            m_inDataDevices.remove(reply);
-        }
-        return;
-    }
-
-    emit errorChanged(reply->errorString());
-}
-
 void QWebdav::provideAuthenication(QNetworkReply *reply, QAuthenticator *authenticator)
 {
-    QVariantHash opts = authenticator->options();
-    qDebug() << "QWebdav::authenticationRequired()  option == " << opts;
+    qCDebug(KDAV2_LOG) << "QWebdav::authenticationRequired()  option == " << authenticator->options();
 
     if (reply == m_authenticator_lastReply) {
         //Avoid endless retries. This will fail with AuthenticationRequiredError
@@ -225,9 +167,9 @@ void QWebdav::provideAuthenication(QNetworkReply *reply, QAuthenticator *authent
     authenticator->setPassword(m_password);
 }
 
-void QWebdav::sslErrors(QNetworkReply *reply, const QList<QSslError> &errors)
+void QWebdav::sslErrors(QNetworkReply *reply, const QList<QSslError> &)
 {
-    qDebug() << "QWebdav::sslErrors()   reply->url == " << reply->url().toString(QUrl::RemoveUserInfo);
+    qCDebug(KDAV2_LOG) << "QWebdav::sslErrors()   reply->url == " << reply->url().toString(QUrl::RemoveUserInfo);
 
     if (m_ignoreSslErrors) {
         // user accepted this SSL certifcate already ==> ignore SSL errors
@@ -241,49 +183,30 @@ QString QWebdav::absolutePath(const QString &relPath)
 
 }
 
-QNetworkReply* QWebdav::createDAVRequest(const QString& method, QNetworkRequest& req, QIODevice* outgoingData)
+QNetworkReply* QWebdav::createDAVRequest(const QString& method, QNetworkRequest& req, const QByteArray& outgoingData)
 {
-    if(outgoingData != 0 && outgoingData->size() !=0) {
-        req.setHeader(QNetworkRequest::ContentLengthHeader, outgoingData->size());
+    if(!outgoingData.isEmpty()) {
+        req.setHeader(QNetworkRequest::ContentLengthHeader, outgoingData.size());
         req.setHeader(QNetworkRequest::ContentTypeHeader, "text/xml; charset=utf-8");
     }
 
-    qDebug() << " QWebdav::createDAVRequest1";
-    qDebug() << "   " << method << " " << req.url().toString();
-    QList<QByteArray> rawHeaderList = req.rawHeaderList();
-    QByteArray rawHeaderItem;
-    foreach(rawHeaderItem, rawHeaderList) {
-        qDebug() << "   " << rawHeaderItem << ": " << req.rawHeader(rawHeaderItem);
+    qCDebug(KDAV2_LOG) << " QWebdav::createDAVRequest\n"
+        << "   " << method << " " << req.url().toString();
+    for (const auto rawHeaderItem : req.rawHeaderList()) {
+        qCDebug(KDAV2_LOG) << "   " << rawHeaderItem << ": " << req.rawHeader(rawHeaderItem);
     }
 
-    return sendCustomRequest(req, method.toLatin1(), outgoingData);
-}
-
-QNetworkReply* QWebdav::createDAVRequest(const QString& method, QNetworkRequest& req, const QByteArray& outgoingData )
-{
-    QBuffer* dataIO = new QBuffer;
-    dataIO->setData(outgoingData);
-    dataIO->open(QIODevice::ReadOnly);
-
-    qDebug() << " QWebdav::createDAVRequest2";
-    qDebug() << "   " << method << " " << req.url().toString();
-    QList<QByteArray> rawHeaderList = req.rawHeaderList();
-    QByteArray rawHeaderItem;
-    foreach(rawHeaderItem, rawHeaderList) {
-        qDebug() << "   " << rawHeaderItem << ": " << req.rawHeader(rawHeaderItem);
+    if (KDAV2_LOG().isDebugEnabled()) {
+        QTextStream stream(stdout, QIODevice::WriteOnly);
+        stream << outgoingData;
     }
 
-    QNetworkReply* reply = createDAVRequest(method, req, dataIO);
-    m_outDataDevices.insert(reply, dataIO);
+    auto reply = sendCustomRequest(req, method.toLatin1(), outgoingData);
+    //For redirects
+    reply->setProperty("requestData", outgoingData);
     return reply;
 }
 
-QNetworkReply* QWebdav::list(const QString& path)
-{
-    qDebug() << "QWebdav::list() path = " << path;
-    return list(path, 1);
-}
-
 QNetworkReply* QWebdav::list(const QString& path, int depth)
 {
     QWebdav::PropNames query;
@@ -346,57 +269,13 @@ QNetworkReply* QWebdav::get(const QString& path, const QMap<QByteArray, QByteArr
         req.setRawHeader(it.key(), it.value());
     }
 
-    qDebug() << "QWebdav::get() url = " << req.url().toString(QUrl::RemoveUserInfo);
+    qCDebug(KDAV2_LOG) << "QWebdav::get() url = " << req.url().toString(QUrl::RemoveUserInfo);
 
     req.setUrl(reqUrl);
 
     return QNetworkAccessManager::get(req);
 }
 
-QNetworkReply* QWebdav::get(const QString& path, QIODevice* data)
-{
-    return get(path, data, 0);
-}
-
-QNetworkReply* QWebdav::get(const QString& path, QIODevice* data, quint64 fromRangeInBytes)
-{
-    QNetworkRequest req;
-
-    QUrl reqUrl(m_baseUrl);
-    reqUrl.setPath(absolutePath(path));
-
-    req.setUrl(reqUrl);
-
-    qDebug() << "QWebdav::get() url = " << req.url().toString(QUrl::RemoveUserInfo);
-
-    if (fromRangeInBytes>0) {
-        // RFC2616 http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html
-        QByteArray fromRange = "bytes=" + QByteArray::number(fromRangeInBytes) + "-";   // byte-ranges-specifier
-        req.setRawHeader("Range",fromRange);
-    }
-
-    QNetworkReply* reply = QNetworkAccessManager::get(req);
-    m_inDataDevices.insert(reply, data);
-    connect(reply, SIGNAL(readyRead()), this, SLOT(replyReadyRead()));
-    connect(reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(replyError(QNetworkReply::NetworkError)));
-
-    return reply;
-}
-
-QNetworkReply* QWebdav::put(const QString& path, QIODevice* data)
-{
-    QNetworkRequest req;
-
-    QUrl reqUrl(m_baseUrl);
-    reqUrl.setPath(absolutePath(path));
-
-    req.setUrl(reqUrl);
-
-    qDebug() << "QWebdav::put() url = " << req.url().toString(QUrl::RemoveUserInfo);
-
-    return QNetworkAccessManager::put(req, data);
-}
-
 QNetworkReply* QWebdav::put(const QString& path, const QByteArray& data, const QMap<QByteArray, QByteArray> &headers)
 {
     QNetworkRequest req;
@@ -409,9 +288,12 @@ QNetworkReply* QWebdav::put(const QString& path, const QByteArray& data, const Q
         req.setRawHeader(it.key(), it.value());
     }
 
-    qDebug() << "QWebdav::put() url = " << req.url().toString(QUrl::RemoveUserInfo);
+    qCDebug(KDAV2_LOG) << "QWebdav::put() url = " << req.url().toString(QUrl::RemoveUserInfo);
 
-    return QNetworkAccessManager::put(req, data);
+    auto reply =  QNetworkAccessManager::put(req, data);
+    reply->setProperty("requestData", data);
+    reply->setProperty("isPut", true);
+    return reply;
 }
 
 
