@@ -66,7 +66,7 @@ class QWEBDAVSHARED_EXPORT QWebdav : public QNetworkAccessManager
 public:
     enum QWebdavConnectionType {HTTP = 1, HTTPS};
 
-    QWebdav(QObject* parent = 0);
+    QWebdav(QObject* parent = nullptr);
     ~QWebdav();
 
     typedef QMap<QString, QMap < QString, QVariant > > PropValues;
@@ -89,16 +89,12 @@ public:
                             int port = 0,
                             bool ignoreSslErrors = true);
 
-    QNetworkReply* list(const QString& path);
-    QNetworkReply* list(const QString& path, int depth);
+    QNetworkReply* list(const QString& path, int depth = 1);
 
     QNetworkReply* search(const QString& path, const QString& query);
 
     QNetworkReply* get(const QString& path, const QMap<QByteArray, QByteArray> &headers);
-    QNetworkReply* get(const QString& path, QIODevice* data);
-    QNetworkReply* get(const QString& path, QIODevice* data, quint64 fromRangeInBytes);
 
-    QNetworkReply* put(const QString& path, QIODevice* data);
     QNetworkReply* put(const QString& path, const QByteArray& data, const QMap<QByteArray, QByteArray> &headers);
 
     QNetworkReply* mkdir(const QString& dir );
@@ -117,28 +113,17 @@ public:
     QNetworkReply* proppatch(const QString& path, const QWebdav::PropValues& props);
     QNetworkReply* proppatch(const QString& path, const QByteArray& query);
 
-Q_SIGNALS:
-    void errorChanged(QString error);
-
 protected Q_SLOTS:
-    void replyReadyRead();
-    void replyFinished(QNetworkReply*);
-    void replyDeleteLater(QNetworkReply*);
-    void replyError(QNetworkReply::NetworkError);
     void provideAuthenication(QNetworkReply* reply, QAuthenticator* authenticator);
     void sslErrors(QNetworkReply *reply,const QList<QSslError> &errors);
 
 protected:
-    QNetworkReply* createDAVRequest(const QString& method, QNetworkRequest& req, QIODevice* outgoingData = 0 );
-    QNetworkReply* createDAVRequest(const QString& method, QNetworkRequest& req, const QByteArray& outgoingData);
+    QNetworkReply* createDAVRequest(const QString& method, QNetworkRequest& req, const QByteArray& outgoingData = {});
 
     //! creates the absolute path from m_rootPath and relPath
     QString absolutePath(const QString &relPath);
 
 private:
-    QMap<QNetworkReply*, QIODevice*> m_outDataDevices;
-    QMap<QNetworkReply*, QIODevice*> m_inDataDevices;
-
     QString m_rootPath;
     QString m_username;
     QString m_password;
