@@ -20,6 +20,7 @@
 
 #include "davmanager.h"
 #include "davprincipalhomesetsfetchjob.h"
+#include "davcollectionfetchjob.h"
 #include "davprotocolbase.h"
 #include "utils.h"
 #include "daverror.h"
@@ -73,7 +74,7 @@ void DavCollectionsFetchJob::principalFetchFinished(KJob *job)
     const DavPrincipalHomeSetsFetchJob *davJob = qobject_cast<DavPrincipalHomeSetsFetchJob *>(job);
 
     if (davJob->error()) {
-        if (davJob->latestResponseCode()) {
+        if (davJob->latestHttpStatusCode()) {
             // If we have a HTTP response code then this may mean that
             // the URL was not a principal URL. Retry as if it were a calendar URL.
             qCDebug(KDAV2_LOG) << "Principal fetch failed, retrying: " << job->errorText();
@@ -81,7 +82,6 @@ void DavCollectionsFetchJob::principalFetchFinished(KJob *job)
         } else {
             // Just give up here.
             setDavError(davJob->davError());
-            setErrorTextFromDavError();
             emitResult();
         }
 
@@ -89,8 +89,10 @@ void DavCollectionsFetchJob::principalFetchFinished(KJob *job)
     }
 
     const QStringList homeSets = davJob->homeSets();
-    qCDebug(KDAV2_LOG) << "Found " << homeSets.size() << " homesets";
-    qCDebug(KDAV2_LOG) << homeSets;
+    qCDebug(KDAV2_LOG) << "Found " << homeSets.size() << " homesets\n " << homeSets;
+
+    //Update the url in case of redirects
+    mUrl.setUrl(davJob->url());
 
     if (homeSets.isEmpty()) {
         // Same as above, retry as if it were a calendar URL.
@@ -118,8 +120,7 @@ void DavCollectionsFetchJob::principalFetchFinished(KJob *job)
 
 void DavCollectionsFetchJob::collectionsFetchFinished(KJob *job)
 {
-    auto davJob = qobject_cast<DavJob *>(job);
-    const int responseCode = davJob->responseCode();
+    auto davJob = static_cast<DavJob *>(job);
 
     if (davJob->error()) {
         if (davJob->url() != mUrl.url()) {
@@ -131,11 +132,7 @@ void DavCollectionsFetchJob::collectionsFetchFinished(KJob *job)
             return;
         }
 
-        setLatestResponseCode(responseCode);
-        setError(ERR_PROBLEM_WITH_REQUEST);
-        setJobErrorText(davJob->errorText());
-        setJobError(davJob->error());
-        setErrorTextFromDavError();
+        setErrorFromJob(davJob);
     } else {
         // For use in the collectionDiscovered() signal
         QUrl _jobUrl = mUrl.url();
@@ -245,6 +242,18 @@ void DavCollectionsFetchJob::collectionsFetchFinished(KJob *job)
                     continue;
                 }
 
+                bool protocolSupportsCTags = DavManager::self()->davProtocol(mUrl.protocol())->supportsCTags();
+                if (protocolSupportsCTags && collection.CTag() == "") {
+                    qCDebug(KDAV2_LOG) << "No CTag found for"
+                        << collection.url().url().toDisplayString()
+                        << "from the home set, trying from the direct URL";
+                    refreshIndividualCollection(collection);
+
+                    responseElement = Utils::nextSiblingElementNS(
+                        responseElement, QStringLiteral("DAV:"), QStringLiteral("response"));
+                    continue;
+                }
+
                 mCollections << collection;
                 Q_EMIT collectionDiscovered(mUrl.protocol(), url.toDisplayString(), jobUrl);
 
@@ -257,6 +266,46 @@ void DavCollectionsFetchJob::collectionsFetchFinished(KJob *job)
     subjobFinished();
 }
 
+// This is a workaroud for Google who doesn't support providing the CTag
+// directly from the home set. We refresh the collections individually from
+// their own URLs, but only if we haven't found a CTag with the home set
+// request.
+void DavCollectionsFetchJob::refreshIndividualCollection(const DavCollection &collection)
+{
+    ++mSubJobCount;
+    auto individualFetchJob = new DavCollectionFetchJob(collection, this);
+    connect(individualFetchJob, &DavCollectionFetchJob::result, this, &DavCollectionsFetchJob::individualCollectionRefreshed);
+    individualFetchJob->start();
+}
+
+void DavCollectionsFetchJob::individualCollectionRefreshed(KJob *job)
+{
+    const auto *davJob = qobject_cast<DavCollectionFetchJob *>(job);
+
+    if (davJob->error()) {
+        if (davJob->latestHttpStatusCode()) {
+            // If we have a HTTP response code then this may mean that
+            // the URL was not a principal URL. Retry as if it were a calendar URL.
+            qCDebug(KDAV2_LOG) << "Individual fetch failed, retrying: " << job->errorText();
+            doCollectionsFetch(mUrl.url());
+        } else {
+            setDavError(davJob->davError());
+            emitResult();
+        }
+        return;
+    }
+
+    qCDebug(KDAV2_LOG) << "Collection"
+        << davJob->collection().url().url().toDisplayString() << "refreshed";
+
+    if (davJob->collection().CTag() == "") {
+        qWarning() << "Collection with an empty CTag";
+    }
+
+    mCollections << davJob->collection();
+    subjobFinished();
+}
+
 void DavCollectionsFetchJob::subjobFinished()
 {
     if (--mSubJobCount == 0) {
